{ lib, config, ... }:
{
  networking = {
    firewall.extraInputRules = ''
      ip saddr { ${config.ips.localSpan}.0/24, 10.100.0.0/24 } tcp dport 2049 accept comment "Accept NFS"
      ip saddr { ${config.ips.pc}, ${config.secrets.lunaIP}, ${config.secrets.cornIP} } tcp dport { 1935, 1945 } accept comment "Accept RTMP"
    '';

    # Nftables configuration only if server is enabled
    nftables.tables.forwarding = {
      family = "ip";
      content = ''
        chain PREROUTING {
          type nat hook prerouting priority dstnat; policy accept;
          tcp dport 2211 dnat to ${config.ips.pc}:22 comment "SSH to PC"
          tcp dport 2222 dnat to 10.100.0.19:22 comment "SSH to Oracle VM"
          
          udp dport { 27005, 27015, 7777 } dnat to ${config.ips.pc} comment "PC Hosted Games"
          
          tcp dport { 48010, 47989, 47984 } dnat to ${config.ips.pc} comment "PC Sunshine TCP"
          udp dport { 47998, 47999, 48000 } dnat to ${config.ips.pc} comment "PC Sunshine UDP"
          
          tcp dport { 38010, 37989, 37984 } dnat to ${config.ips.vm} comment "VM Sunshine TCP"
          udp dport { 37998, 37999, 38000 } dnat to ${config.ips.vm} comment "VM Sunshine UDP"
          
          udp dport { 7790, 7791, 7792 } dnat to ${config.ips.hx} comment "Deus Ex"
          
          ip saddr ${config.secrets.cornIP} tcp dport { 9943, 9944 } dnat to ${config.ips.vm} comment "VM ALVR TCP"
          ip saddr ${config.secrets.cornIP} udp dport { 9943, 9944 } dnat to ${config.ips.vm} comment "VM ALVR UDP"
        }

        chain POSTROUTING {
          type nat hook postrouting priority 100; policy accept;
          oifname "${config.ips.netInt}" masquerade
        }
      '';
    };
  };

  # Enable IP forwarding for the server configuration
  boot.kernel.sysctl."net.ipv4.ip_forward" = lib.mkIf config.system.firewall.server.enable 1;
}
